/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1996, 2012 Oracle and/or its affiliates.  All rights reserved.
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 * $Id$
 */

#include "db_config.h"

#include "db_int.h"

/*
 * __db_getlong --
 *	Return a long value inside of basic parameters.
 *
 * PUBLIC: int __db_getlong
 * PUBLIC:     __P((DB_ENV *, const char *, char *, long, long, long *));
 */
int
__db_getlong(dbenv, progname, p, min, max, storep)
	DB_ENV *dbenv;
	const char *progname;
	char *p;
	long min, max, *storep;
{
	long val;
	char *end;

	__os_set_errno(0);
	val = strtol(p, &end, 10);
	if ((val == LONG_MIN || val == LONG_MAX) &&
	    __os_get_errno() == ERANGE) {
		if (dbenv != NULL)
			dbenv->err(dbenv, ERANGE, "%s", p);
		else
			fprintf(stderr, "%s: %s: %s\n",
			    progname, p, strerror(ERANGE));
		return (ERANGE);
	}
	if (p[0] == '\0' || (end[0] != '\0' && end[0] != '\n')) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0043",
			    "%s: Invalid numeric argument", "%s"), p);
		else
			fprintf(stderr, DB_STR_A("0042",
			    "%s: %s: Invalid numeric argument\n",
			    "%s %s\n"), progname, p);
		return (EINVAL);
	}
	if (val < min) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0045",
			    "%s: Less than minimum value (%ld)",
			    "%s %ld"), p, min);
		else
			fprintf(stderr, DB_STR_A("0044",
			    "%s: %s: Less than minimum value (%ld)\n",
			    "%s %s %ld\n"), progname, p, min);
		return (ERANGE);
	}
	if (val > max) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0047",
			    "%s: Greater than maximum value (%ld)",
			    "%s %ld"), p, max);
		else
			fprintf(stderr, DB_STR_A("0046",
			    "%s: %s: Greater than maximum value (%ld)\n",
			    "%s %s %ld\n"), progname, p, max);
		return (ERANGE);
	}
	*storep = val;
	return (0);
}

/*
 * __db_getulong --
 *	Return an unsigned long value inside of basic parameters.
 *
 * PUBLIC: int __db_getulong
 * PUBLIC:     __P((DB_ENV *, const char *, char *, u_long, u_long, u_long *));
 */
int
__db_getulong(dbenv, progname, p, min, max, storep)
	DB_ENV *dbenv;
	const char *progname;
	char *p;
	u_long min, max, *storep;
{
	u_long val;
	char *end;

	__os_set_errno(0);
	val = strtoul(p, &end, 10);
	if (val == ULONG_MAX && __os_get_errno() == ERANGE) {
#ifndef HAVE_RDS_BUILD
		if (dbenv != NULL)
			dbenv->err(dbenv, ERANGE, "%s", p);
		else
#endif
			fprintf(stderr, "%s: %s: %s\n",
			    progname, p, strerror(ERANGE));
		return (ERANGE);
	}
	if (p[0] == '\0' || (end[0] != '\0' && end[0] != '\n')) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0049",
			    "%s: Invalid numeric argument",
			    "%s"), p);
		else
			fprintf(stderr, DB_STR_A("0048",
			    "%s: %s: Invalid numeric argument\n",
			    "%s %s\n"), progname, p);
		return (EINVAL);
	}
	if (val < min) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0051",
			    "%s: Less than minimum value (%lu)",
			    "%s %lu"), p, min);
		else
			fprintf(stderr, DB_STR_A("0050",
			    "%s: %s: Less than minimum value (%lu)\n",
			    "%s %s %lu\n"), progname, p, min);
		return (ERANGE);
	}

	/*
	 * We allow a 0 to substitute as a max value for ULONG_MAX because
	 * 1) accepting only a 0 value is unlikely to be necessary, and 2)
	 * we don't want callers to have to use ULONG_MAX explicitly, as it
	 * may not exist on all platforms.
	 */
	if (max != 0 && val > max) {
		if (dbenv != NULL)
			dbenv->errx(dbenv, DB_STR_A("0053",
			    "%s: Greater than maximum value (%lu)",
			    "%s %lu"), p, max);
		else
			fprintf(stderr, DB_STR_A("0052",
			    "%s: %s: Greater than maximum value (%lu)\n",
			    "%s %s %lu\n"), progname, p, max);
		return (ERANGE);
	}
	*storep = val;
	return (0);
}
